# This script contains the functions to plot
#
# TO DO: implement category feature in plot!
# Author: Judith Zaugg
###############################################################################

plotHeatMap=function(
		clusterTab=NULL,
		mat,
		label=NULL, 
		category=NULL,
		projectfolder=getwd(),
		xlab='',main='',
		ylab='',
		Tset=Tset,
		plot.intervall=NULL,
		plotfolder=NULL){
	require(lattice)
	require(latticeExtra)
	
# check the inputs
# projectfolder
	projectfolder=gsub('/$','',projectfolder)
	cat(createFolderIfNotExist(projectfolder))
	if(is.null(plotfolder)){
		plotfolder=paste(projectfolder,'plots',sep='/')
		cat(createFolderIfNotExist(plotfolder))
	} 
	cat('used folder',plotfolder,'for plotting\n')
	
# load clusterTab if not supplied	
	clusterTab=loadClusterTab(projectfolder,clusterTabID=clusterTab,FALSE)
	if(class(clusterTab)=='GRanges')
		clusterTab=as.data.frame(elementMetadata(clusterTab))
# clusterTab
	stopifnot(!is.null(clusterTab$clustering),!is.null(clusterTab$silinfo),!is.null(clusterTab$id))
# select clusterTab that are present in mat and sort according to clustering and silinfo
	clusterTab=clusterTab[clusterTab$id%in%c(rownames(mat)),]
	stopifnot(dim(clusterTab)[1]!=0)
	if(!is.null(category)){
		clusterTab$cat=factor(clusterTab[[category]])
	}else{
		clusterTab$cat=0
	}
	clusterTab=clusterTab[order(clusterTab$clustering,clusterTab$silinfo,clusterTab$cat),]
	
# mat: sort according to the clusterTab$id
	mat=mat[match(clusterTab$id,rownames(mat)),]
	if(length(grep('_',colnames(mat)))>0){
		labels=only.text(colnames(mat))
		positions=only.numeric(colnames(mat))
		cat('found',length(unique(labels)),'different factors to be plotted:',unique(labels),'\n')
	}else{
		labels=1
		positions=as.numeric(colnames(mat))
		cat('found only one facter to plot:',label,'\n')
	}
	
# special features used in the plotting function	
	clust=clusterTab$clustering
	no_clust=length(unique(clust))# number of clusters -> needed in axis labels
	cat=clusterTab$cat
	no_cat=length(unique(cat))# number of clusters -> needed in axis labels
	
	lp_L=list()# this is to collect all the plots if return.plot='AllinOne'
	len=length(unique(labels)) # this is the number of factors that are going to be plotted
	matAll=mat
	for(ll in unique(labels)){
		mat=matAll[,only.text(colnames(matAll))==ll]
		x=rep(positions,nrow(mat))
		y=rep(1:nrow(mat),rep(ncol(mat),nrow(mat)))
		z=as.vector(t(mat))
		if(!is.null(plot.intervall)){
			if(length(plot.intervall)==1){
				at=seq(-plot.intervall,plot.intervall,length.out=256)
			} else{
				at=seq(plot.intervall[1],plot.intervall[2],length.out=256)
			}
		}
		else if(quantile(z,0.001,na.rm=T)<0){
			int=max(c(abs(quantile(z,0.01,na.rm=T)),abs(quantile(z,0.99,na.rm=T))))
			at=seq(-int,int,length.out=256)
		}else{
			at=c(seq(quantile(z,0.001,na.rm=T),quantile(z,0.999,na.rm=T),length.out=256))
		}
		colores=colorRampPalette(c('red','white',"darkslateblue"))(length(at)-1)
		z[z>max(at)]=max(at)
		z[z<min(at)]=min(at)
		lp=levelplot(z~x*y,aspect='fill',
				clust=clust,
				cat=cat,
				no_cat=no_cat,
				scales=list(y=list(at=table(clust)/2+c(0,cumsum(table(clust))[-no_clust]),
								labels=names(table(clust)))),
				col.regions=colores,
				at=at,panel=PanFun)
		cat('plotted',ll,'\n')
		lp_L[[ll]]=lp
	}
	heatmapfile=paste(plotfolder,'/',label,'heatmap.png',sep='')
	png(heatmapfile,width=(length(lp_L)*100+150),height=700)
	trellis.par.set(Tset)
	plot(update(do.call('c',lp_L),
					scales=list(alternating=FALSE), 
					xlab=xlab,ylab='',main=main,layout=c(length(lp_L),1)))
	dev.off()
	cat('heatmap has been plotted in',heatmapfile,'\n')
	return(heatmapfile)
}


PanFun=function(x,clust,cat,no_cat,...){
	panel.levelplot(x,...)
	xx=cumsum(table(clust))
	panel.xyplot(0,length(x),type='h',lwd=1,lty=2,col='black')
	for(i in 1:(length(xx))){
		panel.abline(xx[i],0,lwd=2)
		xx_cat=cumsum(table(cat[as.character(clust)==names(table(clust))[i]]))
#		for(j in c(1:length(xx_cat))[-1]){
#			panel.abline(xx_cat[j]+xx[i],0,lwd=1,lty=2)
#		}
	}
}


