# This script contains the functions to plot the properties
#
# Author: Judith Zaugg
###############################################################################

plotProperties=function(
		annoTab=NULL,
		clusterTab=NULL,
		projectfolder=getwd(),
		properties=NULL,
		label=NULL,
		plot.setting=Tset,
		plotfolder=NULL,
		device='png'){
	require(lattice)
	require(latticeExtra)
	
# check the inputs
# projectfolder
	projectfolder=gsub('/$','',projectfolder)
	cat(createFolderIfNotExist(projectfolder))
	if(is.null(plotfolder)){
		plotfolder=paste(projectfolder,'plots',sep='/')
		cat(createFolderIfNotExist(plotfolder))
	} 
	cat('used folder',plotfolder,'for plotting\n')
	plotfolder=gsub('/$','',plotfolder)
	
# load clusterTab if not supplied	
	clusterTab=loadClusterTab(projectfolder,clusterTabID=clusterTab,FALSE)
	if(class(clusterTab)=='GRanges')
		clusterTab=as.data.frame(elementMetadata(clusterTab))
# clusterTab
	stopifnot(!is.null(clusterTab$clustering),!is.null(clusterTab$silinfo),!is.null(clusterTab$id))
	
	
	annoTab=loadAnnoTab(projectfolder,annoTabID=annoTab,FALSE)
	if(class(annoTab)=='GRanges'){
		cat('convert do dataframe\n')
		ids=names(annoTab)
		annoTab=as.data.frame(elementMetadata(annoTab))
		annoTab$id=ids
	}
	if(is.null(properties)){
		properties=loadLogFile(projectfolder,what='added',colum=2)
	}
# sort both tables according to clustering and silinfo	
	clusterTab=clusterTab[order(clusterTab$clustering,clusterTab$silinfo),]
# prepare for plotting the properties
	properties=properties[properties%in%colnames(annoTab)]
	cat('will plot the following properties',properties,'\n')
	clusterTab=clusterTab[!is.na(clusterTab$clustering),]
	if(class(clusterTab$clustering)!='factor'){
		clusterTab$clustering=factor(clusterTab$clustering)
	}
	clust=clusterTab$clustering
	no_clust=length(unique(clust))# number of clusters -> needed in axis labels
	Type='h';Hori=TRUE
	plots_L=list()
	for(property in properties){
		clusterTab$prop=annoTab[match(clusterTab$id,annoTab$id),property]
		#clusterTab=clusterTab[!is.na(clusterTab$prop),]
		print(length(clusterTab$prop))
		print(length(clust))
		plots_L[[property]]=xyplot(1:length(prop)~prop,data=clusterTab,
				type=Type,horizontal=Hori,ylim=c(0,nrow(clusterTab)),
				clust=clust,lwd=1,
				scales=list(y=
								list(at=table(clust)/2+c(0,cumsum(table(clust))[-no_clust]),
										labels=names(table(clust)))),
				ylab='',xlab='',
				panel=PanFun2,col='grey')
		cat('plotted',property,'\n')
	}
	if(device=='png'){
		plotfile=paste(plotfolder,'/',label,'properties.png',sep='')
		png(plotfile,width=(length(plots_L)*100+200),height=700)
	} else {
		plotfile=paste(plotfolder,'/',label,'properties.pdf',sep='')
		pdf(plotfile,width=(length(plots_L)*100+200)/70,height=700/70)
	}
	trellis.par.set(plot.setting)
	plot(update(do.call('c',plots_L),
					scales=list(alternating=FALSE),main='',layout=c(length(plots_L),1)))
	dev.off()
	for(property in properties){
		plotfile2=paste(plotfolder,'/SingleProperty',property,'.png',sep='')
		if(device=='png'){
			png(plotfile2,width=(300),height=200)
		}else{
			pdf(plotfile2,width=(300)/70,height=200/70)
		}
		trellis.par.set(plot.setting)
		plot(update(plots_L[[property]],panel=PanFun3,
						xlab=property,
						ylab='clusters',
						scales=list(x=list(at=levels(clust),labels=levels(clust))),
						ylim=c(min(annoTab[[property]],na.rm=T),max(annoTab[[property]],na.rm=T)),
						xlim=c(min(as.numeric(as.character(clust)),na.rm=T)-1,max(as.numeric(as.character(clust)),na.rm=T)+1)))
		dev.off()
	}
	return(plotfile)
}

PanFun2=function(x,y,clust,pch,cex,col,...){
	xx=cumsum(table(clust))
	xx3=quantile(x,na.rm=T,0.99)
	xx1=median(x,na.rm=T)
	panel.xyplot(x,y,pch=20,cex=0.1,...,col='darkgrey',origin=xx1)
	for(i in 1:(length(xx))){
		sel=names(table(clust))[i]==clust
		xx2=median(x[sel],na.rm=T)
		yy2=xx[i]
		yy1=ifelse(i==1,0,xx[i-1])
		xl=ifelse(x[sel]<xx2,x[sel],xx2)
		xr=ifelse(x[sel]<xx2,xx2,x[sel])
		panel.abline(xx[i],0,lwd=2)
		panel.xyplot(c(xx2,xx2),c(yy1,yy2),type='l',col='red',lty=2,lwd=1)
		wt=wilcox.test(x~sel)
		pv=signif(p.adjust(wt$p.value,method='BH',n=length(xx)),2)
		pos1=ifelse(xx2<xx1,4,2)
		panel.text(xx1,(yy1+yy2)/2,ifelse(pv<0.05,pv,''),cex=0.75,pos=pos1)
		pos2=ifelse(xx2>xx1,4,2)
		panel.text(xx1,(yy1+yy2)/2,ifelse(pv<0.05,"*",''),pos=pos2,col='red',cex=2)
	}
	panel.abline(0,0,lwd=2)
	panel.xyplot(xx1,length(x),type='h',col='black',lwd=1,lty=2)
	
}
PanFun3=function(x,y,clust,pch,cex,col,horizontal,...){
	panel.bwplot(clust,x,...,horizontal=FALSE)
	for(c in unique(clust)){
		sel=c==clust
		wt=wilcox.test(x~sel)
		pv=signif(p.adjust(wt$p.value,method='BH',n=length(unique(clust))),2)
		panel.text(c,quantile(x,0.9,na.rm=T),ifelse(pv<0.05,pv,''),cex=0.75,pos=1)
		panel.text(c,quantile(x,0.9,na.rm=T),ifelse(pv<0.05,'*',''),cex=1.5,pos=3,col='red')
	}
}
